﻿#region

using MVR.FileManagementSecure;
using SimpleJSON;

#endregion

namespace SceneNinja {
    public class PlayerData {

        public readonly string ID;

        private readonly string _label;
        private bool RightSide { get; }
        public string AppearancePath { get; private set; }
        public string PluginsPath { get; private set; }
        public bool SuppressClothing { get; private set; }

        public bool MergeClothing { get; private set; }
        public bool SuppressScale { get; private set; }
        public bool MergePlugins { get; private set; }
        public bool IgnorePluginVersion { get; private set; }


        private JSONStorableBool _suppressClothing;
        private JSONStorableBool _mergeClothing;
        private JSONStorableBool _suppressScale;
        private JSONStorableBool _mergePlugins;
        private JSONStorableBool _ignorePluginVersion;

        private JSONStorableString _appearancePath;
        private JSONStorableString _pluginsPath;

        private JSONStorableString _appearanceText;
        private JSONStorableString _pluginsText;

        private SceneNinja _ninja;

        public PlayerData(string id) {
            ID = id;
            SuppressClothing = true;
            MergeClothing = false;
            SuppressScale = true;
            MergePlugins = true;
            IgnorePluginVersion = true;
            AppearancePath = "";
            PluginsPath = "";
            RightSide = id.Contains("M");
            if (ID.Contains("M")) {
                _label = ID == "M1" ? "Primary Male" : "Secondary Male(s)";
            } else {
                _label = ID == "F1" ? "Primary Female" : "Secondary Female(s)";
            }
        }
        public PlayerData(JSONNode obj) {
            ID = obj["id"];
            _label = obj["label"];
            SuppressClothing = obj["suppressClothing"].AsBool;
            MergeClothing = obj["mergeClothing"].AsBool;
            if (MergeClothing) SuppressClothing = false;
            SuppressScale = obj["suppressScale"].AsBool;
            MergePlugins = obj["mergePlugins"].AsBool;
            IgnorePluginVersion = obj["ignorePluginVersion"].AsBool;
            AppearancePath = obj["appearancePath"];
            PluginsPath = obj["pluginsPath"];
            if (ID.Contains("M")) {
                _label = ID == "M1" ? "Primary Male" : "Secondary Male(s)";
            } else {
                _label = ID == "F1" ? "Primary Female" : "Secondary Female(s)";
            }
            RightSide = ID.Contains("M");
        }

        public void LoadVars() {
            _suppressClothing = new JSONStorableBool(ID + "SuppressClothing", SuppressClothing, BoolUpdated);
            _mergeClothing = new JSONStorableBool(ID + "MergeClothing", MergeClothing, BoolUpdated);
            _suppressScale = new JSONStorableBool(ID + "SuppressScale", SuppressScale, BoolUpdated);
            _mergePlugins = new JSONStorableBool(ID + "MergePlugins", MergePlugins, BoolUpdated);
            _ignorePluginVersion = new JSONStorableBool(ID + "IgnorePluginVersion", IgnorePluginVersion, BoolUpdated);
            _appearancePath = new JSONStorableString(ID + "ApperancePath", AppearancePath, AppearanceUpdated);
            _pluginsPath = new JSONStorableString(ID + "PluginsPath", PluginsPath, PluginUpdated);
            _appearanceText = new JSONStorableString(ID + "ApperanceText", "");
            _pluginsText = new JSONStorableString(ID + "PluginsText", "");


            if (AppearancePath != "") {
                var fileName = AppearancePath;
                var parts = fileName.Split('/');
                var file = parts[parts.Length - 1];
                _appearanceText.valNoCallback = file;
            }

            if (PluginsPath != "") {
                var fileName = PluginsPath;
                var parts = fileName.Split('/');
                var file = parts[parts.Length - 1];
                _pluginsText.valNoCallback = file;
            }
        }


        private void AppearanceUpdated(JSONStorableString js) {

        }

        private void PluginUpdated(JSONStorableString js) {

        }

        private bool ValidatePluginFile(string presetFilePath) {
            if (string.IsNullOrEmpty(presetFilePath)) return false;
            var normalPath = SuperController.singleton.NormalizePath(presetFilePath);

            if (presetFilePath != "" && FileManagerSecure.FileExists(presetFilePath)) {
                var presetPackageName = "";
                if (presetFilePath.Contains(":")) {
                    presetPackageName = presetFilePath.Substring(0, presetFilePath.IndexOf(':'));
                }

                var data = FileManagerSecure.ReadAllText(normalPath);
                var presetJson = JSON.Parse(data);

                if (presetPackageName != "") {
                    var presetJsonString = presetJson.ToString();
                    if (presetJsonString.Contains("SELF:")) {
                        presetJsonString = presetJsonString.Replace("SELF:", presetPackageName + ":");
                        presetJson = JSON.Parse(presetJsonString);
                    }
                }

                var plugs = presetJson["storables"][0]["plugins"];
                if (plugs == null || plugs.Count == 0) {
                    SuperController.LogMessage("Invalid plugin data, returning: " + presetFilePath);
                    return false;
                }
            }

            return true;
        }

        private void BoolUpdated(JSONStorableBool jb) {
            var name = jb.name;
            if (name.Contains("Clothing") && name.Contains("Suppress")) {
                SuppressClothing = jb.val;
                if (SuppressClothing && MergeClothing) {
                    _mergeClothing.valNoCallback = !SuppressClothing;
                    MergeClothing = !SuppressClothing;
                }
            }

            if (name.Contains("Clothing") && name.Contains("Merge")) {
                MergeClothing = jb.val;
                if (SuppressClothing && MergeClothing) {
                    _suppressClothing.valNoCallback = !MergeClothing;
                    SuppressClothing = !MergeClothing;
                }
            }

            if (name.Contains("Scale")) {
                SuppressScale = jb.val;
            }

            if (name.Contains("Merge")) {
                MergePlugins = jb.val;
            }

            if (name.Contains("Ignore")) {
                IgnorePluginVersion = jb.val;
            }
            SuperController.LogMessage("Bool for " + ID + " updated: " + name);
            _ninja.UpdatePlayerData(this);
        }

        public void Initialize(SceneNinja ninja) {
            SuperController.LogMessage("Initializing player: " + ID);
            _ninja = ninja;
            ninja.RegisterBool(_suppressClothing);
            ninja.RegisterBool(_mergeClothing);
            ninja.RegisterBool(_suppressScale);
            ninja.RegisterBool(_mergePlugins);
            ninja.RegisterBool(_ignorePluginVersion);
            ninja.RegisterString(_appearancePath);
            ninja.RegisterString(_pluginsPath);
            ninja.CreateUILabel(_label, RightSide);
            ninja.CreateUIButton(ID + " Appearance", RightSide, SelectAppearanceFile);
            ninja.CreateUIButton("Clear Appearance", RightSide, ClearAppearanceFile);
            ninja.CreateUITextField(_appearanceText, RightSide, 40);
            ninja.CreateUIToggle(_suppressClothing, RightSide, "Suppress Clothing Load");
            ninja.CreateUIToggle(_mergeClothing, RightSide, "Merge Clothing");
            ninja.CreateUIToggle(_suppressScale, RightSide, "Suppress Scale Load");
            ninja.CreateUIButton(ID + " Plugins", RightSide, SelectPluginPreset);
            ninja.CreateUIButton("Clear Plugins", RightSide, ClearPluginsFile);
            ninja.CreateUITextField(_pluginsText, RightSide, 40);
            ninja.CreateUIToggle(_mergePlugins, RightSide, "Merge Load Plugins");
            ninja.CreateUIToggle(_ignorePluginVersion, RightSide, "Ignore Plugin Versions");
        }

        private void ClearPluginsFile() {
            PluginsPath = "";
            _pluginsPath.valNoCallback = "";
            _pluginsText.valNoCallback = "";
            SuperController.LogMessage("Plugins cleared: " + ID);
            _ninja.UpdatePlayerData(this);
        }

        private void ClearAppearanceFile() {
            AppearancePath = "";
            _appearancePath.valNoCallback = "";
            _appearanceText.valNoCallback = "";
            SuperController.LogMessage("Appearance cleared: " + ID);
            _ninja.UpdatePlayerData(this);
        }

        private void SelectPluginPreset() {
            var shortcuts =
                FileManagerSecure.GetShortCutsForDirectory("Custom\\Atom\\Person\\Plugins", false, false, true);
            SuperController.singleton.GetMediaPathDialog(PluginFileSelected, "vap", "Custom\\Atom\\Person\\Plugins",
                false,
                true, false, null, false, shortcuts);
        }

        private void SelectAppearanceFile() {
            var shortcuts =
                FileManagerSecure.GetShortCutsForDirectory("Custom\\Atom\\Person\\Appearance", false, false, true);
            SuperController.singleton.GetMediaPathDialog(AppearanceFileSelected, "vap",
                "Custom\\Atom\\Person\\Appearance", false, true, false, null, false, shortcuts);
        }

        private void PluginFileSelected(string fileName) {
            if (fileName != "") {
                if (ValidatePluginFile(fileName)) {
                    SceneNinja.LogMessage("Plugins selected for " + ID + ": " + fileName);
                    PluginsPath = fileName;
                    var parts = fileName.Split('/');
                    var pFile = parts[parts.Length - 1];
                    _pluginsPath.valNoCallback = fileName;
                    _pluginsText.valNoCallback = pFile;
                    _ninja.UpdatePlayerData(this);
                }
            }
        }

        private void AppearanceFileSelected(string fileName) {
            if (fileName != "") {
                SceneNinja.LogMessage("Appearance selected for " + ID + ": " + fileName);
                AppearancePath = fileName;
                var parts = fileName.Split('/');
                var aFile = parts[parts.Length - 1];
                _appearancePath.valNoCallback = fileName;
                _appearanceText.valNoCallback = aFile;
                _ninja.UpdatePlayerData(this);
            }
        }

        public JSONNode ToJson() {
            var data = new JSONClass {
                { "id", ID },
                { "label", _label },
                { "appearancePath", AppearancePath },
                { "pluginsPath", PluginsPath },
                { "suppressClothing", SuppressClothing.ToString() },
                { "mergeClothing", MergeClothing.ToString() },
                { "suppressScale", SuppressScale.ToString() },
                { "ignorePluginVersion", IgnorePluginVersion.ToString() },
                { "mergePlugins", MergePlugins.ToString() }
            };
            return data;
        }
    }
}
